%humid_troposphere_model
%
% Function which computes and plots the temperature vs height profile
% for humid air (or various relative humidities), and also computes the
% lapse-rate, which is - the gardient of this graph.
%
% Pressure vs height and also dew-point and water boiling point vs height
% are also computed
%
% LAST UPDATED by Andy French 11/11/2014

function humid_troposphere_model

%Set ground level pressure /mbar and temperature /degrees celcius
P0 = 1013.25;
T0 = 15;

%Set known boiling temperature and pressure
Pboil0 = 1013.25;
Tboil0 = 100;

%Set latitude /degrees
lat = 51.5;

%Define range of altitudes /km up to the height of the tropopause
N = 10;
z = linspace(0,11,N);

%Define range of relative humidities
M = 5;
U = linspace(0,1,M);
for m=1:M
    legend_entries{m} = ['U = ',num2str(U(m))];
end

%%

%Determine lapse rates and temperature /deg celcius
T = zeros(N,M);
L = zeros(N,M);
for m=1:M
    for n=1:N
        [P(n,m),T(n,m),L(n,m),Tdew(n,m),Tboil(n,m)] = PT_model(z(n),U(m),P0,T0,Pboil0,Tboil0,lat);
    end
end

%Plot results
figure('color',[1 1 1],'name','lapse rates');

%Lapse rate
subplot(3,1,1);
plot( repmat(z.',[1,M]), L );
xlabel('Altitude /km')
ylabel('Lapse rate/ ^oC /km')
title('Lapse rates for different relative humidities')
legend( legend_entries,'location','eastoutside' );
grid on;

%Temperature
subplot(3,1,2);
plot( repmat(z.',[1,M]), T );
xlabel('Altitude /km')
ylabel('Temperature / ^oC')
legend( legend_entries,'location','eastoutside'  );
grid on;

%Pressure
subplot(3,1,3);
plot( repmat(z.',[1,M]), P );
xlabel('Altitude /km')
ylabel('Pressure / mbar')
legend( legend_entries,'location','eastoutside'  );
grid on;

%Print PNG
print( gcf,'lapse rate','-dpng','-r300');
close(gcf);

%Plot results
figure('color',[1 1 1],'name','lapse rates');

%Temperature
subplot(3,1,1);
plot( repmat(z.',[1,M]), T );
xlabel('Altitude /km')
ylabel('Temperature / ^oC')
legend( legend_entries,'location','eastoutside'  );
grid on;

%Boiling point
subplot(3,1,2);
plot( repmat(z.',[1,M]), Tboil );
xlabel('Altitude /km')
ylabel('Boiling point / ^oC')
legend( legend_entries,'location','eastoutside' );
grid on;

%Dew point
subplot(3,1,3);
plot( repmat(z.',[1,M]), Tdew );
xlabel('Altitude /km')
ylabel('Dew point temperature / ^oC')
legend( legend_entries,'location','eastoutside'  );
grid on;

%Print PNG
print( gcf,'dew and boil','-dpng','-r300');
close(gcf);

%%

%P,T model assuming a constant relative humidity U. Returns
%pressure /mbar and temperature /degrees celcius.
function [P,T,L,Tdew,Tboil] = PT_model(z,U,P0,T0,Pboil0,Tboil0,lat)

%Molar mass of dry air /kg/mol^-1
Md = 0.02896;

%Molar mass of water vapour /kg/mol^-1
Mv = 0.01802;

%Convert altitude z to geopotential altitude and also compute gravitational
%field strength g /ms^-2
[h,g] = geopot(z,lat);

%Specific heat of dry air at constant pressure /Jkg^-1K-1
c_pd = 1003.5;

%Specific gas constant for dry air /Jkg^-1K^-1
Rs = 8.314/Md;

%Latent heat of vaporisation of water /Jkg^-1K^-1 
%(strictly valid at 100 degrees
%celcius and 101,325 Pa ambient pressure. Actually the latent heat
%decreases with temperature, although the gradient of this decrease is
%only really significant for air temperature well beyond those expected
%in the atmosphere).
Hs = 2501000;

%Define height change dh /km
dh = 0.01;

%Iteratively work out temperature, lapse rate and pressure
hh = 0;
P = P0;
T = T0;
Es = vapour_pressure(T0);
r = U*Es/(P0-Es);
L = 1000*g*(1+r*Hs/(Rs*(T+273)))/( c_pd + Hs^2*r*(Mv/Md)/( Rs*(T+273)^2 ) );
Tdew = dew_point( T,U );
Tboil = boiling_point( P, Hs*Mv, Tboil0, Pboil0 );
while hh < h
    hh = hh + dh;
    Es = vapour_pressure(T);
    dP = -Md*g*(P - U*(1-Mv/Md)*Es)/(Rs*Md*(T+273))*dh*1000;
    P = P + dP;
    r = U*Es/(P-Es);
    L = 1000*g*(1+r*Hs/(Rs*(T+273)))/( c_pd + Hs^2*r*(Mv/Md)/( Rs*(T+273)^2 ) );
    T = T - L*dh;
    Tdew = dew_point( T,U );
    Tboil = boiling_point( P, Hs*Mv, Tboil0, Pboil0 );
end

%%

%Partial pressure /mbar of water vapour in (saturated) air
%at temperature T degrees celcius
function Es = vapour_pressure(T)
Es = 6.1121*exp( (18.678 - T/234.5).*(T./( T+257.14 )) );

%%

%Dew point calculation from relative humidity U 
% U=0 means dry air, U=1 means saturated air. T is the ambient temperature
% in degres celcius.
% http://andrew.rsmas.miami.edu/bmcnoldy/Humidity.html

function Tdew = dew_point( T,U )
a = 17.625;
b = 243.04;
A = b*(log(U) +a*T/(b+T));
B = a-log(U)-a*T/(b+T);
Tdew = A/B;

%%

%Boiling point calculator, determined from the Clausius-Clapeyron equation
% approximated when the volume of gas relative to the volume of liquid at
% the same pressure nad temperature (i.e. 'either side' of the phase change)
% is essentialy infinite.
% Boiling temperature Tboil/ degrees celcius.
% P is the vapour pressure of the liquid in mbars
% P0 is the pressure /mbar corresponding to a known boiling point T0 /deg
% Lvap is the latent heat of vaporization /Jmol^-1
function Tboil = boiling_point( P, Lvap, T0, P0 )
T0 = T0 + 273;
R = 8.314;
Tboil = (1/T0 - R*log(P/P0)/Lvap)^-1 - 273;

%%

%Geopotential altitudes /km at latitude lat /deg
function [h,g] = geopot(z,lat)
g0 = 9.80665;
g1 = 9.80616;
a = 0.0026373;
b = 0.0000059;
RE = 6378.1370;
lat = lat*pi/180;
g = g1*( 1-a*cos(2*lat)+b*cos(2*lat)*cos(2*lat) );
h = z*g*RE /( g0*(z+RE) );
 
%End of code