%standard_atmosphere
%
% Plots temperature / degrees celcius and pressure /mbars as described by
% the International Standard Atmosphere model.
%
% Air is assumed to be dry, i.e. humidity efects due to water vapour are
% not modelled.
%
% z is the altitude above sea level in km T is the temperature in degrees
% celcius P is the pressure in milli-bars.
%
% Note: 1 milli-bar is 100 pascals, since 1 bar is 10^5 pascals.
%
% LAST UPDATED by Andy French Nov 2014
%
% ISA data from
% http://en.wikipedia.org/wiki/International_Standard_Atmosphere

function standard_atmosphere

%Define range of altitudes /km
z = linspace(0,80,1000);

%Define latitude /deg
lat = 51.5;

%Compute pressures /mbar and temperatures /degrees celcius
[P,T] = std_atm(z,lat);

%Plot graph and save PNG file
figure('color',[1 1 1],'name','International Standard Atmosphere');
subplot(2,1,1);
plot(z,P);
xlabel('Altitude /km');
ylabel('Pressure / mbar');
title('International standard atmosphere (ISA)');
grid on;
subplot(2,1,2);
plot(z,T);
xlabel('Altitude /km');
ylabel('Temperature / ^oC');
grid on;
print( gcf,'ISA','-dpng','-r300');
close(gcf);

%%

%Compute standard atmosphere model given vector input of altitudes
function [P,T] = std_atm(z,lat)
P = NaN(size(z));
T = NaN(size(z));
for n=1:length(z)
    [P(n),T(n)] = isa_model( z(n),lat );
end

%%

%International standard atmosphere model
function [P,T] = isa_model(z,lat)

%Molar gas constant /Jmol^-1K-1
R = 8.314;

%Molar mass of dry air /kgmol^-1
M = 0.02896;

%Convert altitude into geopotential altitude. This transformation
%corrects for variations in gravitational field strength with altitude.
%Also work out gravitational field strength /ms^-2
[h,g] = geopot(z,lat);

%Define:
% * Lapse rates (degrees celcius /km and altitudes /km
% * tops of different atmospheric layers Altitudes are geometric altitude above mean sea level
% * Temperature /degrees C at base of layer
% * Pressure in mbar at base of layer

if h<=0
    L = 0;
    Pb = NaN;
    Tb = NaN;
    hb = 0;
elseif ( h>=0 ) && ( h < 11 )
    % Troposphere
    L = 6.5;
    Pb = 1013.25;
    Tb = 15.15;
    hb = 0;
elseif ( h >= 11 ) && ( h < 20 )
    %Tropopause
    L = 0;
    Pb = 226.32;
    Tb = -56.5;
    hb = 11;
elseif ( h >= 20 ) && ( h < 32 )
    % Stratosphere (lower)
    L = -1;
    Pb = 54.749;
    Tb = -56.5;
    hb = 20;
elseif ( h>=32 ) && ( h<47 )
    % Stratosphere (upper)
    L = -2.8;
    Pb = 8.6802;
    Tb = -44.5;
    hb = 32;
elseif ( h>=47 ) && ( h<51 )
    % Stratopause
    L = 0;
    Pb = 1.1091;
    Tb = -2.5;
    hb = 47;
elseif ( h>=51 ) && ( h<71 )
    % Mesosphere (lower)
    L = 2.8;
    Pb = 0.66939;
    Tb = -2.5;
    hb = 51;
elseif ( h>=71 ) && ( h<84.852 )
    % Mesosphere (upper)
    L = 2;
    Pb = 3.9564/100;
    Tb = -58.5;
    hb = 71;
else
    %No model as yet for within or above Mesopause
    L = NaN;
    Pb = 0.3734/100;
    Tb = -86.28;
    
    % NOTE Ionosphere extends from about 85km to 600km in altitude
    % At Night: E and F layers
    % During Day: D, E, F layers
    
    %Thermosphere
    
    %Thermopause (or 'exobase')
    
    %Exosphere
    
    %Magnetosphere
end

%Determine temperature /deg celcius & pressure /mbar
if L==0
    T = Tb;
    P = Pb * exp( -M*g*1000*( h - hb )/(R*( Tb +273 )) );
else
    T = Tb - L*( h - hb);
    P = Pb * ( ( 1 - L*( h-hb)/(Tb + 273) )^( M*g/ (L*R/1000) ) );
end

%%

%Geopotential altitudes /km at latitude lat /deg
function [h,g] = geopot(z,lat)
g0 = 9.80665;
g1 = 9.80616;
a = 0.0026373;
b = 0.0000059;
RE = 6378.1370;
lat = lat*pi/180;
g = g1*( 1-a*cos(2*lat)+b*cos(2*lat)*cos(2*lat) );
h = z*g*RE /( g0*(z+RE) );

%%

%Altitude from Geopotential altitude /km and latitude lat /deg
function [z,g] = inv_geopot(h,lat)
g0 = 9.80665;
g1 = 9.80616;
a = 0.0026373;
b = 0.0000059;
RE = 6378.1370;
lat = lat*pi/180;
g = g1*( 1-a*cos(2*lat)+b*cos(2*lat)*cos(2*lat) );
h = h*RE/( RE*g/g0 - h );

%End of code